#include <SD.h>                      // need to include the SD library
//#define SD_ChipSelectPin 53  //example uses hardware SS pin 53 on Mega2560
#define SD_ChipSelectPin 10  //using digital pin 10 on arduino nano 328, can use other pins
#include <TMRpcm.h>           //  also need to include this library...
#include <SPI.h>

TMRpcm tmrpcm;   // create an object for use in this sketch

byte StateM;  // state machine byte
byte MinNum;  // minimum number
byte MaxNum;  // maximum number
byte MaxPred;  // maximum amount of intros
byte StateE;  // error state
byte StateA;  // animation state
unsigned long lastTime=0; // last timestamp - used for animated effects


void setup(){
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(14, INPUT);
  tmrpcm.speakerPin = 9; //5,6,11 or 46 on Mega, 9 on Uno, Nano, etc

  Serial.begin(9600);
  if (!SD.begin(SD_ChipSelectPin)) {  // see if the card is present and can be initialized:
    Serial.println("SD fail");  
    StateM=5;MinNum=1;MaxNum=6;MaxPred=0;
    return;   // don't do anything more if not
  }
  //Serial.println("card initialized.");

  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("kostka.txt");
  StateM=0;
  StateE=0; // no error
  char ReadFromFile;
  // if the file is available, write to it:
  if (dataFile) {
    while ((dataFile.available()) && (StateM<5)) {
      ReadFromFile=dataFile.read();
      switch (StateM) {
       case 0:  // first character = minimal number
        if ((ReadFromFile<'0') || (ReadFromFile>'9')) {
          StateM=5;
          StateE=1;
        } else {
          MinNum=ReadFromFile-'0';
          StateM++;
        }
        break;
       case 1:  // search for end of line (first line)
       case 3:  // search for end of line (second line)
        if (ReadFromFile=='\n')
          StateM++;
        break;
       case 2:  // first character second line = maximal number
        if ((ReadFromFile<'0') || (ReadFromFile>'9')) {
          StateM=5;
          StateE=1;
        } else {
          MaxNum=ReadFromFile-'0';
          StateM++;
        }
        break;
       case 4:  // first character third line = num of pre-speach
        if ((ReadFromFile<'0') || (ReadFromFile>'9')) {
          StateM=5;
          StateE=1;
        } else {
          MaxPred=ReadFromFile-'0';
          StateM++;
        }
        break;
      }
      //Serial.write(ReadFromFile);
    }
    dataFile.close();
  }
  // if the file isn't open, pop up an error:
  else {
   StateE=1;
  }
  tmrpcm.play("intro.wav"); //the sound file "music" will play each time the arduino powers up, or is reset
  if (StateE) {StateM=5;MinNum=1;MaxNum=6;MaxPred=0;} else {StateM=0;} // New start of state machine for main loop
}

void displayx(const byte what){
  switch (what) {
    case 0:
     PORTD = B00000000; // all zeroes
     break;
    case 1:
     PORTD = B00100000; // one in middle
     break;
    case 2:
     PORTD = B00010000; // two
     break;
    case 3:
     PORTD = B00110000; // three
     break;
    case 4:
     PORTD = B00010100; // four
     break;
    case 5:
     PORTD = B00110100; // five
     break;
    case 6:
     PORTD = B00011100; // six
     break;
    case 7:
     PORTD = B00111100; // seven
     break;
    case 8:
     PORTD = B01011100; // eight
     break;
    case 9:
     PORTD = B01111100; // nine = all
     break;

    case 10:
     PORTD = B00000100; // rotate - step 1
     break;
    case 11:
     PORTD = B00001000; // rotate - step 2
     break;
    case 12:
     PORTD = B00010000; // rotate - step 4
     break;
    case 13:
     PORTD = B01000000; // rotate - step 5
     break;

    case 15:
     PORTD = B01101000; // error
     break;

    case 16:
     PORTD = B00000100; // rotate 2 - step 1
     break;
    case 17:
     PORTD = B00001100; // rotate 2 - step 2
     break;
    case 18:
     PORTD = B00011100; // rotate 2 - step 3
     break;
    case 19:
     PORTD = B01011100; // rotate 2 - step 4
     break;
    case 20:
     PORTD = B01011000; // rotate 2 - step 5
     break;
    case 21:
     PORTD = B01010000; // rotate 2 - step 7
     break;
    case 22:
     PORTD = B01000000; // rotate 2 - step 7
     break;
  }
}


/*
 * Statr machine:
 * 0 - new start = display sleeper
 * 1 - button pressed 
 * 2 - delay after button
 * 3 - display number
 * 4 - keep number
 * 5 - display rror
 */

void loop(){  
  char cislo[12] = "cislo_1.wav";
  char hazu[11] =  "hazu_1.wav";
  if (digitalRead(14)==LOW) { // button is pressed
    StateM=1;
  }
  switch (StateM) {
    case 0:
      if (lastTime<millis()) { // time for next animation step
        lastTime+=200;
        if ((StateA>13) || (StateA<10)) {StateA=10;} // just for sure
        displayx(StateA);
        StateA++;
      }
    break;
    case 1:
      if (digitalRead(14)==LOW) { // button is pressed
        random(MinNum, MaxNum+1);
        if (lastTime<millis()) { // time for next animation step
          lastTime+=150;
          if ((StateA>22) || (StateA<16)) {StateA=16;} // just for sure
          displayx(StateA);
          StateA++;
        }
      } else {
        lastTime=millis();
        StateM++;
      }
    break;
    case 2:
      if (lastTime+100<millis()) { // debounce time
        StateM++;
      }
    break;
    case 3:
      byte myNum;
      myNum = random(MinNum, MaxNum+1);
      displayx(myNum);
      cislo[6]='0'+myNum;
      if (MaxPred) {
        hazu[5]='0'+random(1, MaxPred+1);;
        tmrpcm.play(hazu); 
        while (tmrpcm.isPlaying()) {}
        delay(50);
      }
      if (!StateE) {tmrpcm.play(cislo);} 
      StateM++;
    break;
    case 4:

    break;
    case 5:
      displayx(15);
    break;
    default:
    break;
  }
    //if(Serial.read() == 'p'){ //send the letter p over the serial monitor to start playback
    //  tmrpcm.play("cislo_5.wav");
    //}
  //}

}


